@@ -3,7 +3,7 @@ import yaml
 import os
 from .constants import *
 from .logger import Logger
-from .utils import *
+from .dict_utils import *
 from pathlib import Path
 from .exceptions import DSOException
 import imp
@@ -11,7 +11,7 @@ import sys
 
 default_config = {
     'kind': 'dso/application/v1',
-    'version': '1',
+    'version': 1,
     'project': 'default',
     'application': 'default',
     'parameter': {
@@ -155,8 +155,8 @@ class ConfigManager:
         self.check_version()
 
     def check_version(self):
-        if not int(self.merged_config['version']) == default_config['version']:
-            if int(self.merged_config['version']) > default_config['version']:
+        if not int(self.merged_config['version']) == int(default_config['version']):
+            if int(self.merged_config['version']) > int(default_config['version']):
                 Logger.warn(CLI_MESSAGES['DSOConfigNewer'].format(default_config['version'], self.merged_config['version']))
             else:
                 Logger.warn(CLI_MESSAGES['DSOConfigOlder'].format(default_config['version'], self.merged_config['version']))
@@ -172,10 +172,10 @@ class ConfigManager:
             Logger.debug(f"Provider '{provider_id}' has not been loaded. Loading...")
             providerPackagePath = os.path.join(self.install_path, 'provider', provider_id)
             if not os.path.exists(providerPackagePath):
-                raise DSOException(f"No provider found '{provider_id}'.")
+                raise DSOException(f"Provider '{provider_id}' not found.")
             provider = imp.load_package(provider_id, providerPackagePath)
         
-        return getattr(provider, 'default_spec', {})
+        return provider.get_default_spec()
 
 
     def update_merged_config(self):
@@ -283,7 +283,7 @@ class ConfigManager:
         else:
             return None
 
-    def get_template_render_path(self, key=None):
+    def get_template_render_paths(self, key=None):
         result = get_dict_item(self.local_config, ['template', 'render']) or {}
         if not key:
             return result
@@ -354,7 +354,7 @@ class ConfigManager:
                 self.flush_global_config()
                 self.update_merged_config()
             else:
-                raise DSOException(f"'{key}' not found in the local DSO configuratoins.")
+                raise DSOException(f"'{key}' not found in the global DSO configuratoins.")
 
         else:
             Logger.info(f"Deleting '{key}' from the local DSO configurations...")
