@@ -13,17 +13,19 @@ from dsocli.providers import ProviderManager
 from dsocli.parameters import ParameterProvider
 from dsocli.stages import Stages
 from dsocli.constants import *
-from dsocli.utils import set_dict_value
+from dsocli.dict_utils import set_dict_value
 
 
-default_spec = {
+_default_spec = {
 }
 
+def get_default_spec():
+    return _default_spec.copy()
+
 
 session = boto3.session.Session()
 ssm = session.client(
     service_name='ssm',
-    region_name='ap-southeast-2',
 )
 
 
@@ -49,7 +51,7 @@ class AwsSsmParameterProvider(ParameterProvider):
             output += f"/{key}"
         return output
 
-###--------------------------------------------------------------------------------------------
+
 ###--------------------------------------------------------------------------------------------
 
     def get_key_validator(self):
@@ -70,7 +72,7 @@ class AwsSsmParameterProvider(ParameterProvider):
         
         ### check children parameters
         path = self.get_parameter_prefix(project, application, stage, key)
-        # parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['String', 'StringList']},{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
+        # parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['String']},{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
         parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
         if len(parameters['Parameters']) > 0:
             raise DSOException("Parameter key '{0}' is not allowed in the given stage becasue it would overwrite all the parameters in '{0}.*', such as '{0}.{1}'.".format(key,parameters['Parameters'][0]['Name'][len(path)+1:]))
@@ -81,7 +83,7 @@ class AwsSsmParameterProvider(ParameterProvider):
             subKey = '.'.join(scopes[0:n+1])
             path = self.get_parameter_prefix(project, application, stage, subKey)
             Logger.debug(f"Describing parameters: path={path}")
-            # parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Type', 'Values':['String', 'StringList']},{'Key':'Name', 'Values':[path]}])
+            # parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Type', 'Values':['String']},{'Key':'Name', 'Values':[path]}])
             parameters = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
             if len(parameters['Parameters']) > 0:
                 raise DSOException("Parameter key '{0}' is not allowed in the given stage becasue it would overwrite parameter '{1}'.".format(key, subKey))
@@ -90,11 +92,11 @@ class AwsSsmParameterProvider(ParameterProvider):
 
     def locate_parameter(self, project, application, stage, key, uninherited=False):
         Logger.debug(f"Locating SSM parameter: project={project}, application={application}, stage={stage}, key={key}")
-        paths = self.get_ssm_search_paths(project, application, stage, key, uninherited)
+        paths = self.get_hierachy_paths(project, application, stage, key, uninherited)
         Logger.debug(f"SSM paths to search in order: {paths}")
         for path in paths:
             Logger.debug(f"Describing SSM parameters: path={path}")
-            # result = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['String', 'StringList']},{'Key':'Name', 'Values':[path]}])
+            # result = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['String']},{'Key':'Name', 'Values':[path]}])
             result = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
             if len(result['Parameters']) > 0: return result['Parameters']
 
@@ -103,18 +105,21 @@ class AwsSsmParameterProvider(ParameterProvider):
     def load_ssm_path(self, parameters, path, recurisve=True):
         Logger.debug(f"Loading SSM parameters: path={path}")
         p = ssm.get_paginator('get_parameters_by_path')
-        paginator = p.paginate(Path=path, Recursive=recurisve, ParameterFilters=[{'Key': 'Type','Values': ['String', 'StringList']}]).build_full_result()
+        paginator = p.paginate(Path=path, Recursive=recurisve, ParameterFilters=[{'Key': 'Type','Values': ['String']}]).build_full_result()
         for parameter in paginator['Parameters']:
             key = parameter['Name'][len(path)+1:]
-            value = parameter['Value']
             if key in parameters:
                 Logger.warn("Inherited parameter '{0}' overridden.".format(key))
-            parameters[key] = {'Value': value, 'Path': path}
+            parameters[key] = {'Value': parameter['Value'], 
+                                'Path': path,
+                                'Revision': parameter['Version'],
+                                'Date': parameter['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                            }
         return parameters
 
 ###--------------------------------------------------------------------------------------------
 
-    def get_ssm_search_paths(self, project, application, stage, key, uninherited):
+    def get_hierachy_paths(self, project, application, stage, key, uninherited):
         paths = []
         if uninherited:
             paths.append(self.get_parameter_prefix(project, application, stage, key))
@@ -152,7 +157,7 @@ class AwsSsmParameterProvider(ParameterProvider):
 
     def list(self, project, application, stage, uninherited):
         ### construct search path in hierachy with no key specified in reverse order
-        paths = list(reversed(self.get_ssm_search_paths(project, application, stage, None, uninherited)))
+        paths = list(reversed(self.get_hierachy_paths(project, application, stage, None, uninherited)))
         # Logger.debug(f"SSM paths to search in order: {paths}")
         parameters = {}
         for path in paths:
@@ -160,7 +165,12 @@ class AwsSsmParameterProvider(ParameterProvider):
 
         result = {'Parameters': []}
         for item in parameters.items():
-            result['Parameters'].append({'Key': item[0], 'Value': item[1]['Value'], 'Path': item[1]['Path']})
+            result['Parameters'].append({'Key': item[0], 
+                                        'Value': item[1]['Value'], 
+                                        'Path': item[1]['Path'],
+                                        'Revision': item[1]['Revision'],
+                                        'Date': item[1]['Date'],
+                                        })
 
         return result
 
@@ -169,27 +179,78 @@ class AwsSsmParameterProvider(ParameterProvider):
     def add(self, project, application, stage, key, value):
         self.assert_no_scope_overwrites(project, application, stage, key)
         found = self.locate_parameter(project, application, stage, key, True)
-        if found and len(found) > 0 and not found[0]['Type'] in ['String', 'StringList']:
+        if found and len(found) > 0 and not found[0]['Type'] in ['String']:
             raise DSOException(f"Failed to add parameter '{key}' becasue there is already a SSM secret with the same key in the given stage.")
         path = self.get_parameter_prefix(project, application, stage=stage, key=key)
         Logger.info(f"Adding SSM parameter: path={path}")
-        ssm.put_parameter(Name=path, Value=value, Type='String', Overwrite=True)
-        return path
+        response = ssm.put_parameter(Name=path, Value=value, Type='String', Overwrite=True)
+        return {'Key': key, 
+                'Path': path,
+                'Revision': response['Version'],
+                }
 
 ###--------------------------------------------------------------------------------------------
 
-    def get(self, project, application, stage, key):
+    def get(self, project, application, stage, key, revision=None):
         found = self.locate_parameter(project, application, stage, key)
-        if not found or len(found) == 0:
+        if not found:
             raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            # if len(found) > 1:
-            #     Logger.warn(f"More than one parameter found at '{found[0]['Name']}'. The first one taken, and the rest were discarded.")
-            if not found[0]['Type'] in ['String', 'StringList']:
+            if not found[0]['Type'] in ['String']:
                 raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Getting SSM parameter: path={found[0]['Name']}")
-        result = ssm.get_parameter(Name=found[0]['Name'], WithDecryption=True)
-        return result['Parameter']['Value']
+        response = ssm.get_parameter_history(Name=found[0]['Name'])
+        parameters = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
+        if revision is None:
+            ### get the latest revision
+            result = {
+                    'Revision': parameters[0]['Version'],
+                    'Value': parameters[0]['Value'],
+                    'Date': parameters[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                    'User': parameters[0]['LastModifiedUser'],
+                    'Key': key, 
+                    'Path': found[0]['Name'],
+                    }
+        else:
+            ### get specific revision
+            parameters = [x for x in parameters if str(x['Version']) == str(revision)]
+            if not parameters:
+                raise DSOException(f"Revision '{revision}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
+            result = {
+                    'Revision': parameters[0]['Version'],
+                    'Value': parameters[0]['Value'],
+                    'Date': parameters[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                    'User': parameters[0]['LastModifiedUser'],
+                    'Key': key, 
+                    'Path': found[0]['Name'],
+                    }
+
+        return result
+
+###--------------------------------------------------------------------------------------------
+
+    def history(self, project, application, stage, key):
+        found = self.locate_parameter(project, application, stage, key)
+        if not found:
+            raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
+        else:
+            if not found[0]['Type'] in ['String']:
+                raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
+        Logger.info(f"Getting SSM parameter: path={found[0]['Name']}")
+        response = ssm.get_parameter_history(Name=found[0]['Name'])
+        parameters = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
+        result = { "Revisions":
+            [{
+                'Revision': parameter['Version'],
+                'Value': parameter['Value'],
+                'Date': parameter['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                'User': parameter['LastModifiedUser'],
+                'Key': key,
+                'Path': found[0]['Name'],
+            } for parameter in parameters]
+        }
+
+        return result
 
 ###--------------------------------------------------------------------------------------------
 
@@ -201,12 +262,11 @@ class AwsSsmParameterProvider(ParameterProvider):
         else:
             # if len(found) > 1:
             #     Logger.warn(f"More than one parameter found at '{found[0]['Name']}'. The first one taken, and the rest were discarded.")
-            if not found[0]['Type'] in ['String', 'StringList']:
+            if not found[0]['Type'] in ['String']:
                 raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Deleting SSM parameter: path={found[0]['Name']}")
-        ssm.delete_parameter(Name=found[0]['Name'])
-        return found[0]['Name']
-
+        response = ssm.delete_parameter(Name=found[0]['Name'])
+        return {'Key': key, 'Path': found[0]['Name']}
 
 ###--------------------------------------------------------------------------------------------
 ###--------------------------------------------------------------------------------------------
