@@ -13,17 +13,19 @@ from dsocli.providers import ProviderManager
 from dsocli.secrets import SecretProvider
 from dsocli.stages import Stages
 from dsocli.constants import *
-from dsocli.utils import set_dict_value
+from dsocli.dict_utils import set_dict_value
 
 
-default_spec = {
+_default_spec = {
 }
 
+def get_default_spec():
+    return _default_spec.copy()
+
 
 session = boto3.session.Session()
 ssm = session.client(
     service_name='ssm',
-    region_name='ap-southeast-2',
 )
 
 
@@ -91,7 +93,7 @@ class AwsSsmSecretProvider(SecretProvider):
 
     def locate_secret(self, project, application, stage, key, uninherited=False):
         Logger.debug(f"Locating SSM secret: project={project}, application={application}, stage={stage}, key={key}")
-        paths = self.get_ssm_search_paths(project, application, stage, key, uninherited)
+        paths = self.get_hierachy_paths(project, application, stage, key, uninherited)
         Logger.debug(f"SSM paths to search in order: {paths}")
         for path in paths:
             Logger.debug(f"Describing SSM secrets: path={path}")
@@ -110,12 +112,16 @@ class AwsSsmSecretProvider(SecretProvider):
             value = secret['Value']
             if key in secrets:
                 Logger.warn("Inherited secret '{0}' overridden.".format(key))
-            secrets[key] = {'Value': value, 'Path': path}
+            secrets[key] = {'Value': secret['Value'], 
+                                'Path': path,
+                                'Revision': secret['Version'],
+                                'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                            }
         return secrets
 
 ###--------------------------------------------------------------------------------------------
 
-    def get_ssm_search_paths(self, project, application, stage, key, uninherited):
+    def get_hierachy_paths(self, project, application, stage, key, uninherited):
         paths = []
         if uninherited:
             paths.append(self.get_secret_prefix(project, application, stage, key))
@@ -153,7 +159,7 @@ class AwsSsmSecretProvider(SecretProvider):
 
     def list(self, project, application, stage, uninherited, decrypt):
         ### construct search path in hierachy with no key specified in reverse order
-        paths = list(reversed(self.get_ssm_search_paths(project, application, stage, None, uninherited)))
+        paths = list(reversed(self.get_hierachy_paths(project, application, stage, None, uninherited)))
         # Logger.debug(f"SSM paths to search in order: {paths}")
         secrets = {}
         for path in paths:
@@ -161,7 +167,12 @@ class AwsSsmSecretProvider(SecretProvider):
 
         result = {'Secrets': []}
         for item in secrets.items():
-            result['Secrets'].append({'Key': item[0], 'Value': item[1]['Value'], 'Path': item[1]['Path']})
+            result['Secrets'].append({'Key': item[0], 
+                                        'Value': item[1]['Value'], 
+                                        'Path': item[1]['Path'],
+                                        'Revision': item[1]['Revision'],
+                                        'Date': item[1]['Date'],
+                                        })
 
         return result
 
@@ -174,23 +185,74 @@ class AwsSsmSecretProvider(SecretProvider):
             raise DSOException(f"Failed to add secret '{key}' becasue there is already a SSM parameter with the same key in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         path = self.get_secret_prefix(project, application, stage=stage, key=key)
         Logger.info(f"Adding SSM secret: path={path}")
-        ssm.put_parameter(Name=path, Value=value, Type='SecureString', Overwrite=True)
-        return path
+        response = ssm.put_parameter(Name=path, Value=value, Type='SecureString', Overwrite=True)
+        return {'Key': key, 
+                'Path': path,
+                'Revision': response['Version'],
+                }
 
 ###--------------------------------------------------------------------------------------------
 
-    def get(self, project, application, stage, key):
+    def get(self, project, application, stage, key, revision=None):
         found = self.locate_secret(project, application, stage, key)
-        if not found or len(found) == 0:
-                raise DSOException(f"Secret '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
+        if not found:
+            raise DSOException(f"Secret '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
-            # if len(found) > 1:
-            #     Logger.warn(f"More than one secret found at '{found[0]['Name']}'. The first one taken, and the rest were discarded.")
             if not found[0]['Type'] in ['SecureString']:
-                raise DSOException(f"Secret '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
+                raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Getting SSM secret: path={found[0]['Name']}")
-        result = ssm.get_parameter(Name=found[0]['Name'], WithDecryption=True)
-        return result['Parameter']['Value']
+        response = ssm.get_parameter_history(Name=found[0]['Name'], WithDecryption=True)
+        secrets = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
+        if revision is None:
+            ### get the latest revision
+            result = {
+                    'Revision': secrets[0]['Version'],
+                    'Value': secrets[0]['Value'],
+                    'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                    'User': secrets[0]['LastModifiedUser'],
+                    'Key': key, 
+                    'Path': found[0]['Name'],
+                    }
+        else:
+            ### get specific revision
+            secrets = [x for x in secrets if str(x['Version']) == str(revision)]
+            if not secrets:
+                raise DSOException(f"Revision '{revision}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
+            result = {
+                    'Revision': secrets[0]['Version'],
+                    'Value': secrets[0]['Value'],
+                    'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                    'User': secrets[0]['LastModifiedUser'],
+                    'Key': key, 
+                    'Path': found[0]['Name'],
+                    }
+
+        return result
+
+###--------------------------------------------------------------------------------------------
+
+    def history(self, project, application, stage, key):
+        found = self.locate_secret(project, application, stage, key)
+        if not found:
+            raise DSOException(f"Secret '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
+        else:
+            if not found[0]['Type'] in ['SecureString']:
+                raise DSOException(f"Parameter '{key}' not found: project={project}, application={application}, stage={Stages.shorten(stage)}")
+        Logger.info(f"Getting SSM secret: path={found[0]['Name']}")
+        response = ssm.get_parameter_history(Name=found[0]['Name'], WithDecryption=True)
+        secrets = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
+        result = { "Revisions":
+            [{
+                'Revision': secret['Version'],
+                'Value': secret['Value'],
+                'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
+                'User': secret['LastModifiedUser'],
+                'Key': key,
+                'Path': found[0]['Name'],
+            } for secret in secrets]
+        }
+
+        return result
 
 ###--------------------------------------------------------------------------------------------
 
@@ -206,7 +268,7 @@ class AwsSsmSecretProvider(SecretProvider):
                 raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Deleting SSM secret: path={found[0]['Name']}")
         ssm.delete_parameter(Name=found[0]['Name'])
-        return found[0]['Name']
+        return {'Key': key, 'Path': found[0]['Name']}
 
 ###--------------------------------------------------------------------------------------------
 ###--------------------------------------------------------------------------------------------
