@@ -11,7 +11,8 @@ from .providers import StoreProvider, ProviderManager
 from .parameters import Parameters
 from .secrets import Secrets
 from .logger import Logger
-from .utils import clean_directory, merge_dicts, deflatten_dict
+from .file_utils import clean_directory
+from .dict_utils import merge_dicts, deflatten_dict
 from .exceptions import DSOException
 from .stages import Stages
 
@@ -34,37 +35,54 @@ class TemplatesClass():
     def default_render_path(self):
         return Config.working_dir
 
-    def list(self):
+    def list(self, uninherited):
         project = Config.project
         application = Config.application
         provider = ProviderManager.TemplateProvider()
         Logger.info(f"Start listing templates: project={project}, application={application}")
-        return provider.list(project, application)
+        renderPaths = Config.get_template_render_paths()
+        renderBasePath = Templates.default_render_path
+        response = provider.list(project, application, uninherited)
+        for template in response['Templates']:
+            key = template['Key']
+            if key in renderPaths:
+                renderPath = renderPaths[key]
+                if not (renderPath == '.' or renderPath.startswith(f'.{os.sep}')):
+                    renderPath = os.path.join('./', renderPath)
+            else:
+                renderPath = os.path.join(renderBasePath, key)
+
+            template['RenderTo'] = renderPath
+        
+        return response
 
     def add(self, key, content, render_path):
         self.validate_key(key)
         project = Config.project
         application = Config.application
         provider = ProviderManager.TemplateProvider()
-        if render_path:
-            # if '..' in render_path:
-            #     raise DSOException(CLI_MESSAGES['InvalidRenderPath'].format(render_path))
-            if render_path == '.' or render_path == './':
-                render_path = os.path.join(self.default_render_path, key)
-            else:
-                if '**' in render_path:
-                    render_path = render_path.replace('**', os.path.dirname(key))
-                if '*' in render_path:
-                    render_path = render_path.replace('*', os.path.basename(key))
-            if not re.match('^[A-Za-z0-9._/$-]+$', render_path):
-                raise DSOException(CLI_MESSAGES['InvalidRenderPath'].format(render_path))
-            if os.path.isdir(render_path):
-                raise DSOException(CLI_MESSAGES['InvalidRenderPathExistingDir'].format(render_path))
-            if os.path.isabs(render_path):
-                Logger.warn(f"Render path {render_path} is not releative to the application root.")
+        # if '..' in render_path:
+        #     raise DSOException(CLI_MESSAGES['InvalidRenderPath'].format(render_path))
+        if render_path == '.' or render_path == './':
+            render_path = os.path.join(self.default_render_path, key)
+        else:
+            if '**' in render_path:
+                render_path = render_path.replace('**', os.path.dirname(key))
+            if '*' in render_path:
+                render_path = render_path.replace('*', os.path.basename(key))
+        if not re.match('^[A-Za-z0-9._/$-]+$', render_path):
+            raise DSOException(CLI_MESSAGES['InvalidRenderPath'].format(render_path))
+        if os.path.isdir(render_path):
+            raise DSOException(CLI_MESSAGES['InvalidRenderPathExistingDir'].format(render_path))
+        if os.path.isabs(render_path):
+            Logger.warn(f"Render path {render_path} is not releative to the application root.")
+        else:
+            if not render_path.startswith(f".{os.sep}"):
+                render_path = os.path.join(f".{os.sep}", render_path)
         try:
             Logger.info(f"Start adding template: project={project}, application={application}, key={key}, render_path={render_path}")
             result = provider.add(project, application, key, content)
+            result['RenderTo'] = render_path
         finally:
             if os.path.abspath(render_path) == os.path.abspath(os.path.join(self.default_render_path, key)):
                 Config.unregister_template_custom_render_path(key)
@@ -80,14 +98,14 @@ class TemplatesClass():
         Logger.info(f"Start getting template: project={project}, application={application}, key={key}")
         return provider.get(project, application, key)
 
-    def delete(self, key, recursive):
+    def delete(self, key):
         # self.validate_key(key)
         project = Config.project
         application = Config.application
         provider = ProviderManager.TemplateProvider()
         Logger.info(f"Start deleting template: project={project}, application={application}, key={key}")
         try:
-            result = provider.delete(project, application, key, recursive)
+            result = provider.delete(project, application, key)
         finally:
             Config.unregister_template_custom_render_path(key)
         return result
@@ -118,8 +136,8 @@ class TemplatesClass():
 
         provider = ProviderManager.TemplateProvider()
         Logger.info(CLI_MESSAGES['LoadingTemplates'])
-        templates = provider.list(project, application)
-        renderPaths = Config.get_template_render_path()
+        templates = provider.list(project, application, uninherited=False)['Templates']
+        renderPaths = Config.get_template_render_paths()
 
         jinja_env = jinja2.Environment(undefined=jinja2.StrictUndefined)
 
@@ -161,7 +179,7 @@ class TemplatesClass():
             with open(renderPath, 'w', encoding='utf-8') as f:
                 f.write(renderedContent)
             
-            rendered.append(renderPath)
+            rendered.append({'Key':key, 'RenderTo': renderPath})
 
         return rendered
 
